#include <ClickEncoder.h>
#include <AD9833.h>
#include <TimerOne.h>
#include "LedControl.h"
#include <math.h>

#define FNC_PIN 10 

boolean up = false;
boolean down = false;
boolean middle = false;
boolean edit_mode = false;

ClickEncoder *encoder;
int16_t last, value;

AD9833 gen(FNC_PIN);
LedControl lc=LedControl(6,7,5,1);
unsigned long delaytime=50;

int frequency[8] = {0,0,0,1,0,0,0,0};
int index = 0;

int i,prev,edit;
int first_run = 1;

const int tr_selector = A3;
const int sq_selector = A4;
const int buzzer = 8;
const int sine = 4;
const int squa = 3;
const int tria = 2;

void setup() {
  gen.Begin();
  gen.ApplySignal(SINE_WAVE,REG0,1000);
  gen.EnableOutput(true);
  encoder = new ClickEncoder(A1, A0, A2);
  encoder->setAccelerationEnabled(false); 

  Timer1.initialize(1000);
  Timer1.attachInterrupt(timerIsr); 
  
  last = encoder->getValue();

  Serial.begin(9600);

  lc.shutdown(0,false);
  lc.setIntensity(0,15);
  lc.clearDisplay(0);
  
  pinMode(tr_selector, INPUT);
  pinMode(sq_selector, INPUT);
  pinMode(buzzer, OUTPUT);
  pinMode(sine, OUTPUT);
  pinMode(squa, OUTPUT);
  pinMode(tria, OUTPUT);
}

void loop() {

  prev = index;
  edit = 0;
  
  readRotaryEncoder();

   ClickEncoder::Button b = encoder->getButton();
   if (b != ClickEncoder::Open) {
   switch (b) {
      case ClickEncoder::Clicked:
         middle=true;
        break;
    }
  }   

   if(middle == true)
   {
    digitalWrite(buzzer, HIGH); 
   }
  
   if(middle == true && edit_mode == false)
   {
    Serial.println("Click");
    edit_mode = true;
    middle = false;
   }
   else if(middle == true && edit_mode == true)
   {
    edit_mode = false;
    middle = false;
   }

   if(up == true && edit_mode == false)
   {
    Serial.println("up");
    if(index < 7)
    {
      index++;
    }
    else
    {
      index = 0;
    }
    up = false;
   }
   else if(up == true && edit_mode == true)
   {
    if(frequency[index] == 9)
    {
      frequency[index] = 0;
    }
    else
    {
      frequency[index]++;
    }
    
    edit = 1;
    up = false;
   }

   if(down == true && edit_mode == false)
   {
    Serial.println("down");
    if(index > 0)
    {
      index--;
    }
    else
    {
      index = 7;
    }
    down = false;
   }
   else if(down == true && edit_mode == true)
   {
    if(frequency[index] == 0)
    {
      frequency[index] = 9;
    }
    else
    {
      frequency[index]--;
    }
    edit = 1;
    down = false;
   }


// Display the current frequency along with pointer here
  if(prev != index || edit == 1 || first_run == 1)
  {
      for(i=0;i<8;i++)
      {
       if(index == i)
       {
         lc.setChar(0,i,'.',true);
         lc.setDigit(0,i,frequency[i],true); 
       }
       else
       {
         lc.setDigit(0,i,frequency[i],false);
       } 
      }

      if(digitalRead(sq_selector) == HIGH)
  {
    gen.ApplySignal(SQUARE_WAVE,REG0,decimal(frequency));
    digitalWrite(sine, LOW);
    digitalWrite(squa, HIGH);
    digitalWrite(tria, LOW);
  }
  else if(digitalRead(tr_selector) == HIGH)
  {
    gen.ApplySignal(TRIANGLE_WAVE,REG0,decimal(frequency));
    digitalWrite(sine, LOW);
    digitalWrite(squa, LOW);
    digitalWrite(tria, HIGH);
  }
  else
  {
    gen.ApplySignal(SINE_WAVE,REG0,decimal(frequency));
    digitalWrite(sine, HIGH);
    digitalWrite(squa, LOW);
    digitalWrite(tria, LOW);
  }
  }
  Serial.print(frequency[7]);
  Serial.print(frequency[6]);
  Serial.print(frequency[5]);
  Serial.print(frequency[4]);
  Serial.print(frequency[3]);
  Serial.print(frequency[2]);
  Serial.print(frequency[1]);
  Serial.print(frequency[0]);
  //Serial.println("done");
  //Serial.println(decimal(frequency));
  
  
  first_run = 0;

  //delay(1000);
  digitalWrite(buzzer, LOW);

  
}
 
long int decimal(int a[8])
{
  long int b;
  b = a[7]*pow(10,7) + a[6]*pow(10,6) + a[5]*pow(10,5) + a[4]*pow(10,4) + a[3]*pow(10,3) + a[2]*pow(10,2) + a[1]*10 + a[0];
  return(b);
}
  
void timerIsr() 
{
  encoder->service(); 
}



void readRotaryEncoder()
{
  value += encoder->getValue();
  
  if (value/2 > last) {
    last = value/2;
    down = true;
    delay(25);
  }else   if (value/2 < last) {
    last = value/2;
    up = true;
    delay(25);
  }
}
